#ifndef DRAWGDIX_H
#define DRAWGDIX_H
             
/*******************************************************************

GDI Helper Classes, MFC Version, by G. Bavestrelli, Techint S.p.A. 
Any feedback is welcome, you can contact me at:
                 giovanni.bavestrelli@pomini.it

These classes handle the creation of, selection in, deselection 
from and deletion of all major GDI objects in/from their DC

Just by creating an object of one of the following types you are 
sure that all necessary creating/selecting/deselecting/deleting is 
done automatically

These classes are fully defined in this file, there is no CPP file, 
meaning all members are inline and so there should be very little 
performance overhead

If you use these classes you can forget about deselecting objects
from the DC and also deleting them unless you create them yourself

These classes handle Pens, Fonts, Brushes and Bitmaps.. but just in
their most common usage (as far as creation is concerned). 

It is easy to extend these classes to accomodate your own 
preferences. You should modify them so they work with your own
options and defaults. 

You should use only one object of each type of class in each one
of your functions, and use that object to modify the relative 
resource. For example, only one CSelPen object in any function.
Keep in mind that CSelStock could interfere with CSelFont,
CSelBrush or CSelPen if used together with them in the wrong way.

These classes don't throw exceptions, they use assertions instead.
If you prefer to use exceptions, feel free to modify the code.


EXAMPLES:

1) Creating and selecting a red pen in the DC,
   also deselecting and deleting it

   CSelPen AnyObjectName(DC,RGB(256,0,0)); // this does it all!

2) Selecting an existing pen in the DC, 
   also deselecting but not deleting it

   CSelPen AnyName(DC,pPen); // pPen is a pointer to the CPen

3) Creating and selecting a font in the DC, 
   also deselecting and deleting it

   CSelFont CallItAnything(DC,8,"Arial"); // this does it all!

4) Selecting an existing bitmap in the DC, 
   also deselecting it but not deleting it

   CSelBitmap Whatever(DC,pBitmap); // pBitmap is a CBitmap * 

5) Creating a bitmap compatible to pCompDC
   and selecting it into DC

   CSelBitmap Peter(DC,pCompDC,width,height); // Easy!

*******************************************************************/

//******************************************************************
// Base class, stores CDC * 
//******************************************************************

class CSelect
{
  protected:
   CDC * const m_pDC;
   CSelect(CDC * pDC):m_pDC(pDC)    { ASSERT(m_pDC);}
   virtual ~CSelect() {}

  private:

   // Disable copying
   operator = (const CSelect & d); 
   CSelect(const CSelect &); 
};

//******************************************************************
// Class for Stock Objects
//******************************************************************

class CSelStock : public CSelect
{
   CGdiObject * m_pOldObj;

  public:
    
   CSelStock(CDC * pDC)
     :CSelect(pDC),m_pOldObj(NULL)
   {
   }
    
   CSelStock(CDC * pDC, int index)
     :CSelect(pDC)
   {
      VERIFY(m_pOldObj=m_pDC->SelectStockObject(index)); 
   }

   ~CSelStock()
   { 
      if (m_pOldObj) VERIFY(m_pDC->SelectObject(m_pOldObj));
   }

   void Select(int index)
   { 
      CGdiObject * old=m_pDC->SelectStockObject(index); ASSERT(old); 
      if (!m_pOldObj) m_pOldObj=old;
   }

   void Restore()
   {
      if (m_pOldObj) VERIFY(m_pDC->SelectObject(m_pOldObj));
      m_pOldObj=NULL;
   }

   CGdiObject * Old() const { return m_pOldObj; }
};
                            
//******************************************************************
// Pens
//******************************************************************

class CSelPen  : public CSelect
{
   CPen   m_NewPen;
   CPen * m_pOldPen;

  public:
        
   CSelPen(CDC * pDC)
     :CSelect(pDC),m_pOldPen(NULL){}  
        
   CSelPen(CDC * pDC, COLORREF col, int sty=PS_SOLID, int wid=0)
     :CSelect(pDC)
   {
      VERIFY(m_NewPen.CreatePen(sty,wid,col));
      VERIFY(m_pOldPen=m_pDC->SelectObject(&m_NewPen));
   }

   CSelPen(CDC * pDC, CPen * pPen)
     :CSelect(pDC)
   { 
      ASSERT(pPen);
      VERIFY(m_pOldPen=m_pDC->SelectObject(pPen)); 
   }

   ~CSelPen() 
   { 
      if (m_pOldPen) VERIFY(m_pDC->SelectObject(m_pOldPen)); 
   }

   void Select(CPen * pPen)
   { 
      ASSERT(pPen);
      ASSERT(pPen!=&m_NewPen);
      CPen * old=m_pDC->SelectObject(pPen); ASSERT(old);
      if (!m_pOldPen) m_pOldPen=old;
      m_NewPen.DeleteObject();
   }

   void Select(COLORREF col, int sty=PS_SOLID, int wid=0)
   { 
      if (m_pOldPen) Select(m_pOldPen);
      VERIFY(m_NewPen.CreatePen(sty,wid,col));
      VERIFY(m_pOldPen=m_pDC->SelectObject(&m_NewPen));
   }

   void Restore()
   { 
      if (m_pOldPen) VERIFY(m_pDC->SelectObject(m_pOldPen)); 
      m_pOldPen=NULL;
      m_NewPen.DeleteObject();
   }

   CPen * Old() const    { return m_pOldPen; }
};

//******************************************************************
// Brushes
//******************************************************************

class CSelBrush  : public CSelect
{
    CBrush   m_NewBrush;
    CBrush * m_pOldBrush;

public:               
   CSelBrush(CDC * pDC)
     :CSelect(pDC),m_pOldBrush(NULL)
   {
   }

   // Solid brush
   CSelBrush(CDC * pDC, COLORREF crColor) 
     :CSelect(pDC)
   { 
      VERIFY(m_NewBrush.CreateSolidBrush(crColor));
      VERIFY(m_pOldBrush=m_pDC->SelectObject(&m_NewBrush));
   }

   // Hatch brush
   CSelBrush(CDC * pDC, int index,COLORREF crColor) 
     :CSelect(pDC)
   { 
      VERIFY(m_NewBrush.CreateHatchBrush(index,crColor));
      VERIFY(m_pOldBrush=m_pDC->SelectObject(&m_NewBrush));
   }

   // Pattern brush
   CSelBrush(CDC * pDC, CBitmap * pBitmap) 
     :CSelect(pDC)
   { 
      ASSERT(pBitmap);
      VERIFY(m_NewBrush.CreatePatternBrush(pBitmap));
      VERIFY(m_pOldBrush=m_pDC->SelectObject(&m_NewBrush));
   }

   // DIB Pattern brush
   CSelBrush(CDC * pDC,HGLOBAL hPackedDib,UINT usage)
     :CSelect(pDC)
   {
      VERIFY(m_NewBrush.CreateDIBPatternBrush(hPackedDib,usage));
      VERIFY(m_pOldBrush=m_pDC->SelectObject(&m_NewBrush));
   }

   CSelBrush(CDC * pDC, CBrush * pBrush) 
     :CSelect(pDC)
   { 
      ASSERT(pBrush);
      VERIFY(m_pOldBrush=m_pDC->SelectObject(pBrush)); 
   }

   ~CSelBrush()
   {
      if (m_pOldBrush) VERIFY(m_pDC->SelectObject(m_pOldBrush)); 
   }

   void Select(CBrush * pBrush) 
   { 
      ASSERT(pBrush);
      ASSERT(pBrush!=&m_NewBrush);
      CBrush * old=m_pDC->SelectObject(pBrush); ASSERT(old);
      if (!m_pOldBrush) m_pOldBrush=old;
      m_NewBrush.DeleteObject();
   }

   // Solid brush
   void Select(COLORREF col) 
   { 
      if (m_pOldBrush) Select(m_pOldBrush);
      VERIFY(m_NewBrush.CreateSolidBrush(col));
      VERIFY(m_pOldBrush=m_pDC->SelectObject(&m_NewBrush));
   }

   // Hatch brush
   void Select(int index,COLORREF col) 
   { 
      if (m_pOldBrush) Select(m_pOldBrush);
      VERIFY(m_NewBrush.CreateHatchBrush(index,col));
      VERIFY(m_pOldBrush=m_pDC->SelectObject(&m_NewBrush));
   }

   // Pattern brush
   void Select(CBitmap * pBitmap) 
   { 
      if (m_pOldBrush) Select(m_pOldBrush);
      VERIFY(m_NewBrush.CreatePatternBrush(pBitmap));
      VERIFY(m_pOldBrush=m_pDC->SelectObject(&m_NewBrush));
   }

   // DIB Pattern brush
   void Select(HGLOBAL hPackedDib,UINT usage)
   {
      if (m_pOldBrush) Select(m_pOldBrush);
      VERIFY(m_NewBrush.CreateDIBPatternBrush(hPackedDib,usage));
      VERIFY(m_pOldBrush=m_pDC->SelectObject(&m_NewBrush));
   }

   void Restore()
   { 
      if (m_pOldBrush) VERIFY(m_pDC->SelectObject(m_pOldBrush)); 
      m_pOldBrush=NULL;
      m_NewBrush.DeleteObject();
   }

   CBrush * Old() const    { return m_pOldBrush; }       
};



//******************************************************************
// My own font with different constructor and creation function
//******************************************************************

class CMyFont  : public CFont
{
  public:
   CMyFont(){}
   CMyFont(CDC * pDC,int size,LPCTSTR face=NULL,BOOL bold=0,
           BOOL italic=0, BOOL underlined=0,BOOL fixed=0,
           BOOL hiquality=0,int angleindegrees=0)
   {  
      VERIFY(MyCreateFont(pDC,size,face,bold,italic,underlined,
                          fixed,hiquality,angleindegrees));
   }
   BOOL MyCreateFont(CDC * pDC,int size,LPCTSTR face=NULL,
                     BOOL bold=0,BOOL italic=0,BOOL underlined=0,
                     BOOL fixed=0,BOOL hiquality=0,
                     int angleindegrees=0) 
   {  
      ASSERT(pDC);
      CSize Size(0,MulDiv(size,pDC->GetDeviceCaps(LOGPIXELSY),72));
      pDC->DPtoLP(&Size);
      return CreateFont(-abs(Size.cy),0,10*angleindegrees,0,
                bold?FW_BOLD:FW_NORMAL,BYTE(italic),
                BYTE(underlined),0,ANSI_CHARSET,
                OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,
                BYTE(hiquality?PROOF_QUALITY:DEFAULT_QUALITY),
                BYTE(fixed?FF_MODERN|FIXED_PITCH:
                           FF_SWISS|VARIABLE_PITCH),
                face && *face?face:NULL); 
   }
};


//******************************************************************
// Fonts
//******************************************************************


class CSelFont  : public CSelect
{
   CMyFont m_NewFont;
   CFont * m_pOldFont;             

public:           
   
   CSelFont(CDC * pDC)
    :CSelect(pDC),m_pOldFont(NULL)
   {
   }

   CSelFont(CDC * pDC,int size,LPCTSTR face=NULL,BOOL bold=0,
            BOOL italic=0, BOOL underlined=0,BOOL fixed=0,
            BOOL hiquality=0,int angleindegrees=0) 
    :CSelect(pDC)
   { 
      VERIFY(m_NewFont.MyCreateFont(m_pDC,size,face,bold,italic,
             underlined,fixed,hiquality,angleindegrees));
      VERIFY(m_pOldFont=m_pDC->SelectObject(&m_NewFont)); 
   }

   CSelFont(CDC * pDC, CFont * pFont)
     :CSelect(pDC)
   { 
      ASSERT(pFont);
      VERIFY(m_pOldFont=m_pDC->SelectObject(pFont)); 
   }

   ~CSelFont()
   { 
      if (m_pOldFont) VERIFY(m_pDC->SelectObject(m_pOldFont)); 
   }

   void Select(CFont * pFont)
   { 
      ASSERT(pFont);
      ASSERT(pFont!=&m_NewFont);
      CFont * old=m_pDC->SelectObject(pFont); ASSERT(old);
      if (!m_pOldFont) m_pOldFont=old;
      m_NewFont.DeleteObject();
   }

   void Select(int size,LPCTSTR face=NULL,BOOL bold=0,
               BOOL italic=0,BOOL underlined=0,BOOL fixed=0,
               BOOL hiquality=0,int angleindegrees=0) 
   { 
      if (m_pOldFont) Select(m_pOldFont);
      VERIFY(m_NewFont.MyCreateFont(m_pDC,size,face,bold,italic,
             underlined,fixed,hiquality,angleindegrees));
      VERIFY(m_pOldFont=m_pDC->SelectObject(&m_NewFont)); 
   }

   void Restore()
   {
      if (m_pOldFont) VERIFY(m_pDC->SelectObject(m_pOldFont)); 
      m_pOldFont=NULL;
      m_NewFont.DeleteObject();
   }

   CFont * Old() const  { return m_pOldFont; }
};


//******************************************************************
// Bitmaps
//******************************************************************

class CSelBitmap  : public CSelect
{
   CBitmap   m_NewBmp;
   CBitmap * m_pOldBmp;

public:            
   CSelBitmap(CDC * pDC)
     :CSelect(pDC),m_pOldBmp(NULL)
   {
   }

   CSelBitmap(CDC * SelectInDC, CDC * CompatibleToDC, int w, int h)
       :CSelect(SelectInDC)
   { 
      ASSERT(CompatibleToDC);
      VERIFY(m_NewBmp.CreateCompatibleBitmap(CompatibleToDC,w,h));
      VERIFY(m_pOldBmp=m_pDC->SelectObject(&m_NewBmp)); 
   }

   CSelBitmap(CDC * pDC, CBitmap * pBitmap)
       :CSelect(pDC)
   { 
      ASSERT(pBitmap);
      VERIFY(m_pOldBmp=m_pDC->SelectObject(pBitmap)); 
   }

   ~CSelBitmap()
   { 
      if (m_pOldBmp) VERIFY(m_pDC->SelectObject(m_pOldBmp)); 
   }

   void Select(CBitmap * pBitmap)
   {
      ASSERT(pBitmap);
      ASSERT(pBitmap!=&m_NewBmp);
      CBitmap * old=m_pDC->SelectObject(pBitmap);  ASSERT(old);
      if (!m_pOldBmp) m_pOldBmp=old;
      m_NewBmp.DeleteObject();
   }

   void Select(CDC * CompatibleToDC, int w, int h)
   {
      ASSERT(CompatibleToDC);
      if (m_pOldBmp) Select(m_pOldBmp);
      VERIFY(m_NewBmp.CreateCompatibleBitmap(CompatibleToDC,w,h));
      VERIFY(m_pOldBmp=m_pDC->SelectObject(&m_NewBmp)); 
   }

   void Restore()
   {
      if (m_pOldBmp) VERIFY(m_pDC->SelectObject(m_pOldBmp)); 
      m_pOldBmp=NULL;
      m_NewBmp.DeleteObject();
   }

   CBitmap * Old() const { return m_pOldBmp; }
};

// This class is a bit different
class CSelPalette  : public CSelect
{
   // You need your own palette, use CPalette
   CPalette * oldPalette;             

public:           
   
   CSelPalette(CDC * pDC)
     :CSelect(pDC),oldPalette(NULL)
   {
   }

   CSelPalette(CDC * pDC, CPalette * pPalette, 
               BOOL ForceBackground=FALSE, BOOL Realize=TRUE)
     :CSelect(pDC)
   { 
      ASSERT(pPalette);
      ASSERT(m_pDC->GetDeviceCaps(RASTERCAPS)&RC_PALETTE);
      VERIFY(oldPalette=m_pDC->SelectPalette(pPalette,
                                           ForceBackground)); 
      if (Realize) m_pDC->RealizePalette();
   }

   ~CSelPalette()
   { 
      if (oldPalette) 
         VERIFY(m_pDC->SelectPalette(oldPalette,FALSE)); 
   }

   UINT Select(CPalette * pPalette, 
               BOOL ForceBackground=FALSE, BOOL Realize=TRUE)
   { 
      ASSERT(pPalette);
      ASSERT(m_pDC->GetDeviceCaps(RASTERCAPS)&RC_PALETTE);
      CPalette * old=m_pDC->SelectPalette(pPalette,ForceBackground);
      ASSERT(old);
      if (!oldPalette) oldPalette=old;
      if (Realize) return m_pDC->RealizePalette();
      else         return 0;
   }

   void Restore()
   {  
      if (oldPalette)
          VERIFY(m_pDC->SelectPalette(oldPalette,FALSE)); 
      oldPalette=NULL;
   }

   CPalette * Old() const  { return oldPalette; }
};


//******************************************************************
// Set and restore other characteristics of the DC (no GDI objects)
//******************************************************************


class CSelROP2 : public CSelect
{
   int m_OldRop;

  public:

   CSelROP2(CDC * pDC,int drawMode)
     :CSelect(pDC)
   { 
      VERIFY(m_OldRop=m_pDC->SetROP2(drawMode));
   }

   CSelROP2(CDC * pDC)
     :CSelect(pDC)
   { 
      VERIFY(m_OldRop=m_pDC->GetROP2()); 
   }

   ~CSelROP2()
   { 
      if (m_OldRop) VERIFY(m_pDC->SetROP2(m_OldRop)); 
   }

   void Select(int drawmode)
   { 
      int old=m_pDC->SetROP2(drawmode); ASSERT(old); 
      if (!m_OldRop) m_OldRop=old;
   }

   void Restore()
   { 
      if (m_OldRop) VERIFY(m_pDC->SetROP2(m_OldRop)); 
      m_OldRop=0;
   }

   int Old() const { return m_OldRop; }
};


class CSelBkMode : public CSelect
{
   int m_OldBkMode;

  public:

   CSelBkMode(CDC * pDC,int BkMode)
     :CSelect(pDC)
   { 
      VERIFY(m_OldBkMode=m_pDC->SetBkMode(BkMode));
   }

   CSelBkMode(CDC * pDC)
     :CSelect(pDC)
   { 
      VERIFY(m_OldBkMode=m_pDC->GetBkMode()); 
   }

   ~CSelBkMode()
   { 
      if (m_OldBkMode) VERIFY(m_pDC->SetBkMode(m_OldBkMode)); 
   }

   void Select(int bkmode)
   {
      int old=m_pDC->SetBkMode(bkmode); ASSERT(old); 
      if (!m_OldBkMode) m_OldBkMode=old;
   }

   void Restore()
   { 
     if (m_OldBkMode) VERIFY(m_pDC->SetBkMode(m_OldBkMode)); 
     m_OldBkMode=0;
   }

   int Old() const { return m_OldBkMode; }
};



class CSelBkColor : public CSelect
{
   COLORREF m_OldBkColor;

  public:

   CSelBkColor(CDC * pDC,COLORREF BkColor)
     :CSelect(pDC)
   { 
      m_OldBkColor=m_pDC->SetBkColor(BkColor);
   }

   CSelBkColor(CDC * pDC)
     :CSelect(pDC)
   { 
      m_OldBkColor=m_pDC->GetBkColor(); 
   }

   ~CSelBkColor()
   { 
      m_pDC->SetBkColor(m_OldBkColor); 
   }

   void Select(COLORREF color)
   {  
      m_pDC->SetBkColor(color); 
   }

   void Restore()
   {
      m_pDC->SetBkColor(m_OldBkColor); 
   }

   COLORREF Old() const { return m_OldBkColor; }
};


class CSelTextColor : public CSelect
{
   COLORREF m_OldTextColor;

  public:

   CSelTextColor(CDC * pDC,COLORREF TextColor)
     :CSelect(pDC)
   {
      m_OldTextColor=m_pDC->SetTextColor(TextColor);
   }

   CSelTextColor(CDC * pDC)
     :CSelect(pDC)
   {
      m_OldTextColor=m_pDC->GetTextColor(); 
   }

   ~CSelTextColor()
   {
      m_pDC->SetTextColor(m_OldTextColor); 
   }

   void Select(COLORREF color)
   {
      m_pDC->SetTextColor(color); 
   }

   void Restore()
   { 
      m_pDC->SetTextColor(m_OldTextColor); 
   }

   COLORREF Old() const { return m_OldTextColor; }
};


class CSelTextAlign : public CSelect
{
   UINT m_OldTextAlign;

  public:

   CSelTextAlign(CDC * pDC,UINT TextAlign)
     :CSelect(pDC)
   {
      m_OldTextAlign=m_pDC->SetTextAlign(TextAlign);
   }

   CSelTextAlign(CDC * pDC)
     :CSelect(pDC)
   {
      m_OldTextAlign=m_pDC->GetTextAlign(); 
   }

   ~CSelTextAlign()
   {
      m_pDC->SetTextAlign(m_OldTextAlign); 
   }

   void Select(UINT align)
   {
      m_pDC->SetTextAlign(align); 
   }

   void Restore()
   { 
      m_pDC->SetTextAlign(m_OldTextAlign); 
   }

   UINT Old() const { return m_OldTextAlign; }
};


class CSelMapMode : public CSelect
{
   int m_OldMapMode;

  public:

   CSelMapMode(CDC * pDC,int MapMode)
     :CSelect(pDC)
   {
      VERIFY(m_OldMapMode=m_pDC->SetMapMode(MapMode));
   }

   CSelMapMode(CDC * pDC)
     :CSelect(pDC)
   {
      VERIFY(m_OldMapMode=m_pDC->GetMapMode()); 
   }

   ~CSelMapMode()
   { 
      if (m_OldMapMode) VERIFY(m_pDC->SetMapMode(m_OldMapMode)); 
   }

   void Select(int mode)
   { 
      int old=m_pDC->SetMapMode(mode); ASSERT(old); 
      if (!m_OldMapMode) m_OldMapMode=old;
   }

   void Restore()
   {
      if (m_OldMapMode) VERIFY(m_pDC->SetMapMode(m_OldMapMode)); 
      m_OldMapMode=0;
   }

   UINT Old() const { return m_OldMapMode; }
};

//******************************************************************
// Class for saving the complete state of the DC
//******************************************************************

class CSaveDC : public CSelect
{
   int m_SavedDC;

  public:

   CSaveDC(CDC * pDC)
     :CSelect(pDC)
   { 
      VERIFY(m_SavedDC=m_pDC->SaveDC()); 
   }

   ~CSaveDC()
   { 
      if (m_SavedDC) VERIFY(m_pDC->RestoreDC(m_SavedDC)); 
   }

   void Restore()
   {
      if (m_SavedDC) VERIFY(m_pDC->RestoreDC(m_SavedDC)); 
      m_SavedDC=0;
   }

   int Old() const { return m_SavedDC; }
};


#endif
