#ifndef DRAWGDIW_H
#define DRAWGDIW_H
             
/*******************************************************************

GDI Helper Classes, SDK Version, by G. Bavestrelli, Techint S.p.A. 
Any feedback is welcome, you can contact me at:
                 giovanni.bavestrelli@pomini.it

These classes have been adapted from the ones that use MFC.
I created these classes for the SDK readers of WDJ.
I use the MFC ones (in DRAWGDIX.H) so these are not tested as much.
If you find problems with them, please let me know.

These classes handle the creation of, selection in, deselection 
from and deletion of all major GDI objects in/from their DC

Just by creating an object of one of the following types you are 
sure that all necessary creating/selecting/deselecting/deleting is 
done automatically

These classes are fully defined in this file, there is no CPP file, 
meaning all members are inline and so there should be very little 
performance overhead

If you use these classes you can forget about deselecting objects
from the DC and also deleting them unless you create them yourself

These classes handle Pens, Fonts, Brushes and Bitmaps.. but just in
their most common usage (as far as creation is concerned). 

It is easy to extend these classes to accomodate your own 
preferences. You should modify them so they work with your own
options and defaults. 

You should use only one object of each type of class in each one
scope, and use that object to modify the relative resource.
For example, only one CSelPen object between any two {}
Keep in mind that CSelStock could interfere with CSelFont,
CSelBrush or CSelPen if used together with them in the wrong way.

These classes don't throw exceptions, they use assertions instead.
But if you don't use MFC, you'll have to #define ASSERT and VERIFY.
If you prefer to use exceptions, feel free to modify the code.


EXAMPLES:

1) Creating and selecting a red pen in the DC,
   also deselecting and deleting it in the destructor

   CSelPen AnyObjectName(hDC,RGB(256,0,0)); // this does it all!

2) Selecting an existing pen in the DC, 
   also deselecting but not deleting it

   CSelPen AnyName(hDC,hPen); // hPen is an HPEN 

3) Creating and selecting a font in the DC, 
   also deselecting and deleting it at the end.

   CSelFont CallItAnything(hDC,8,"Arial"); // this does it all!

4) Selecting an existing bitmap in the DC hDC, 
   also deselecting it but not deleting it

   CSelBitmap Whatever(hDC,hBitmap); // hBitmap is an HBITMAP 

5) Creating a bitmap compatible to the DC hCompDC and selecting
   it into hDC. It will be deselected and deleted too.

   CSelBitmap Peter(hDC,hCompDC,width,height); // Easy!

*******************************************************************/
   
//******************************************************************
// Base class, stores HDC (protected) 
//******************************************************************

class CSelect
{
  protected:

   const HDC m_hDC; 

   CSelect(HDC hDC):m_hDC(hDC)    { ASSERT(m_hDC);}
   virtual ~CSelect(){}

  private:

   // Disable copying
   operator = (const CSelect & d); 
   CSelect(const CSelect &); 
};

//******************************************************************
// Base class for GDI objects (protected)
//******************************************************************

class CSelGdiObj : public CSelect
{

 protected:

   HGDIOBJ m_hNewObj;
   HGDIOBJ m_hOldObj;

   CSelGdiObj(HDC hDC)
        :CSelect(hDC),m_hNewObj(NULL),m_hOldObj(NULL)
   {
   }

   // Change the GDI object in the DC
   void SelectObj(HGDIOBJ hObj)
   { 
      ASSERT(hObj);
      ASSERT(hObj!=m_hNewObj); 
      HGDIOBJ old=SelectObject(m_hDC,hObj); 
      ASSERT(old);
      if (!m_hOldObj) m_hOldObj=old;
      if (m_hNewObj) VERIFY(DeleteObject(m_hNewObj));
      m_hNewObj=NULL;
   }

 public:

   ~CSelGdiObj() 
   { 
      if (m_hOldObj) VERIFY(SelectObject(m_hDC,m_hOldObj)); 
      if (m_hNewObj) VERIFY(DeleteObject(m_hNewObj));
   }

   // Restore previous GDI object
   void Restore()
   { 
      if (m_hOldObj) VERIFY(SelectObject(m_hDC,m_hOldObj)); 
      m_hOldObj=NULL;
      if (m_hNewObj) VERIFY(DeleteObject(m_hNewObj));
      m_hNewObj=NULL;
   }
};

//******************************************************************
// Class for Stock Objects
//******************************************************************

class CSelStock : public CSelect
{
   HGDIOBJ m_hOldObject;

  public:
    
   CSelStock(HDC hDC):CSelect(hDC),m_hOldObject(NULL){}
    
   CSelStock(HDC hDC, int index)
     :CSelect(hDC)
   {
     VERIFY(m_hOldObject=SelectObject(m_hDC,GetStockObject(index))); 
   }

   ~CSelStock()
   { 
     if (m_hOldObject) VERIFY(SelectObject(m_hDC,m_hOldObject));
   }

   void Select(int index)
   { 
     HGDIOBJ old=SelectObject(m_hDC,GetStockObject(index)); 
     ASSERT(old); 
     if (!m_hOldObject) m_hOldObject=old;
   }

   void Restore()
   {
     if (m_hOldObject) VERIFY(SelectObject(m_hDC,m_hOldObject));
     m_hOldObject=NULL;
   }

   HGDIOBJ Old() const { return m_hOldObject; }
};
                            
//******************************************************************
// Pens
//******************************************************************

class CSelPen  : public CSelGdiObj
{
  public:
        
   CSelPen(HDC hDC):CSelGdiObj(hDC){}  
        
   CSelPen(HDC hDC, COLORREF color, int style=PS_SOLID, int width=0)
     :CSelGdiObj(hDC)
   {
      VERIFY(m_hNewObj=CreatePen(style,width,color));
      VERIFY(m_hOldObj=SelectObject(m_hDC,m_hNewObj));
   }

   CSelPen(HDC hDC, HPEN hPen)
     :CSelGdiObj(hDC)
   { 
      ASSERT(hPen);
      VERIFY(m_hOldObj=SelectObject(m_hDC,hPen)); 
   }

   void Select(HPEN hPen)
   {
      ASSERT(hPen);  
      SelectObj(hPen);
   }

   void Select(COLORREF color, int style=PS_SOLID, int width=0)
   { 
      if (m_hOldObj) SelectObj(m_hOldObj);
      VERIFY(m_hNewObj=CreatePen(style,width,color));
      VERIFY(m_hOldObj=SelectObject(m_hDC,m_hNewObj));
   }

   HPEN Old() const { return (HPEN)m_hOldObj; }
};

//******************************************************************
// Brushes
//******************************************************************

class CSelBrush  : public CSelGdiObj
{

public:               
   CSelBrush(HDC hDC):CSelGdiObj(hDC){}

   // Solid brush
   CSelBrush(HDC hDC, COLORREF crColor) 
     :CSelGdiObj(hDC)
   { 
      VERIFY(m_hNewObj=CreateSolidBrush(crColor));
      VERIFY(m_hOldObj=SelectObject(m_hDC,m_hNewObj));
   }

   // Hatch brush
   CSelBrush(HDC hDC, int index,COLORREF crColor) 
     :CSelGdiObj(hDC)
   { 
      VERIFY(m_hNewObj=CreateHatchBrush(index,crColor));
      VERIFY(m_hOldObj=SelectObject(m_hDC,m_hNewObj));
   }

   // Pattern brush
   CSelBrush(HDC hDC, HBITMAP hBitmap) 
     :CSelGdiObj(hDC)
   { 
      ASSERT(hBitmap);
      VERIFY(m_hNewObj=CreatePatternBrush(hBitmap));
      VERIFY(m_hOldObj=SelectObject(m_hDC,m_hNewObj));
   }

   // DIB Pattern brush
   CSelBrush(HDC hDC,HGLOBAL hPackedDib,UINT usage)
     :CSelGdiObj(hDC)
   {
      ASSERT(hPackedDib); 
      VERIFY(m_hNewObj=CreateDIBPatternBrush(hPackedDib,usage));
      VERIFY(m_hOldObj=SelectObject(m_hDC,m_hNewObj));
   }

   CSelBrush(HDC hDC, HBRUSH hBrush) 
     :CSelGdiObj(hDC)
   { 
      ASSERT(hBrush);
      VERIFY(m_hOldObj=SelectObject(m_hDC,hBrush)); 
   }

   void Select(HBRUSH hBrush) 
   { 
      ASSERT(hBrush);  
      SelectObj(hBrush);
   }

   // Solid brush
   void Select(COLORREF col) 
   { 
      if (m_hOldObj) SelectObj(m_hOldObj);
      VERIFY(m_hNewObj=CreateSolidBrush(col));
      VERIFY(m_hOldObj=SelectObject(m_hDC,m_hNewObj));
   }

   // Hatch brush
   void Select(int index,COLORREF col) 
   { 
      if (m_hOldObj) SelectObj(m_hOldObj);
      VERIFY(m_hNewObj=CreateHatchBrush(index,col));
      VERIFY(m_hOldObj=SelectObject(m_hDC,m_hNewObj));
   }

   // Pattern brush
   void Select(HBITMAP hBitmap) 
   { 
      ASSERT(hBitmap);
      if (m_hOldObj) SelectObj(m_hOldObj);
      VERIFY(m_hNewObj=CreatePatternBrush(hBitmap));
      VERIFY(m_hOldObj=SelectObject(m_hDC,m_hNewObj));
   }

   // DIB Pattern brush
   void Select(HGLOBAL hPackedDib,UINT usage)
   {
      ASSERT(hPackedDib); 
      if (m_hOldObj) SelectObj(m_hOldObj);
      VERIFY(m_hNewObj=CreateDIBPatternBrush(hPackedDib,usage));
      VERIFY(m_hOldObj=SelectObject(m_hDC,m_hNewObj));
   }

   HBRUSH Old() const { return (HBRUSH)m_hOldObj; }
};


//******************************************************************
// My own font creation helper functions
//******************************************************************

inline void MyDPtoLP(HDC hDC,LPSIZE lpSize)
{
    SIZE sizeWinExt; VERIFY(::GetWindowExtEx(hDC,&sizeWinExt));
    SIZE sizeVpExt;  VERIFY(::GetViewportExtEx(hDC,&sizeVpExt));
    lpSize->cx = MulDiv(lpSize->cx, abs(sizeWinExt.cx),
                                    abs(sizeVpExt.cx));
    lpSize->cy = MulDiv(lpSize->cy, abs(sizeWinExt.cy), 
                                    abs(sizeVpExt.cy));
}

inline HFONT MyCreateFont(HDC hDC,int size,LPCTSTR face=NULL,
                     BOOL bold=0,BOOL italic=0,BOOL underlined=0,
                     BOOL fixed=0,BOOL hiquality=0,
                     int angleindegrees=0) 
{  
      ASSERT(hDC);

      SIZE Size;
      Size.cx=0;
      Size.cy=MulDiv(size,GetDeviceCaps(hDC,LOGPIXELSY),72);
      MyDPtoLP(hDC,&Size);

      return CreateFont(-abs(Size.cy),0,10*angleindegrees,0,
                bold?FW_BOLD:FW_NORMAL,BYTE(italic),
                BYTE(underlined),0,ANSI_CHARSET,
                OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,
                BYTE(hiquality?PROOF_QUALITY:DEFAULT_QUALITY),
                BYTE(fixed?FF_MODERN|FIXED_PITCH:
                           FF_SWISS|VARIABLE_PITCH),
                face && *face?face:NULL); 
}


//******************************************************************
// Fonts
//******************************************************************

class CSelFont  : public CSelGdiObj
{

public:           
   
   CSelFont(HDC hDC):CSelGdiObj(hDC){}

   CSelFont(HDC hDC,int size,LPCTSTR face=NULL,BOOL bold=0,
            BOOL italic=0, BOOL underlined=0,BOOL fixed=0,
            BOOL hiquality=0,int angleindegrees=0) 
    :CSelGdiObj(hDC)
   { 
      VERIFY(m_hNewObj=MyCreateFont(m_hDC,size,face,bold,italic,
                     underlined,fixed,hiquality,angleindegrees)); 
      VERIFY(m_hOldObj=SelectObject(m_hDC,m_hNewObj)); 
   }

   CSelFont(HDC hDC, HFONT hFont)
       :CSelGdiObj(hDC)
   { 
      ASSERT(hFont);
      VERIFY(m_hOldObj=SelectObject(m_hDC,hFont)); 
   }

   void Select(HFONT hFont)
   {
      ASSERT(hFont);  
      SelectObj(hFont);
   }

   void Select(int size,LPCTSTR face=NULL,BOOL bold=0,
               BOOL italic=0,BOOL underlined=0,BOOL fixed=0,
               BOOL hiquality=0,int angleindegrees=0) 
   { 
      if (m_hOldObj) SelectObj(m_hOldObj);
      VERIFY(m_hNewObj=MyCreateFont(m_hDC,size,face,bold,italic,
                     underlined,fixed,hiquality,angleindegrees)); 
      VERIFY(m_hOldObj=SelectObject(m_hDC,m_hNewObj)); 
   }

   HFONT Old() const { return (HFONT)m_hOldObj; }
};


//******************************************************************
// Bitmaps
//******************************************************************

class CSelBitmap  : public CSelGdiObj
{

public:            
   CSelBitmap(HDC hDC):CSelGdiObj(hDC){}

   CSelBitmap(HDC SelectInDC, HDC CompatibleToDC, int w, int h)
       :CSelGdiObj(SelectInDC)
   { 
      ASSERT(CompatibleToDC);
      VERIFY(m_hNewObj=CreateCompatibleBitmap(CompatibleToDC,w,h));
      VERIFY(m_hOldObj=SelectObject(m_hDC,m_hNewObj)); 
   }

   CSelBitmap(HDC hDC, HBITMAP hBitmap)
       :CSelGdiObj(hDC)
   { 
      ASSERT(hBitmap);
      VERIFY(m_hOldObj=SelectObject(m_hDC,hBitmap)); 
   }

   void Select(HBITMAP hBitmap)
   {
      ASSERT(hBitmap);
      SelectObj(hBitmap);
   }

   void Select(HDC CompatibleToDC, int w, int h)
   {
      ASSERT(CompatibleToDC);
      if (m_hOldObj) SelectObj(m_hOldObj);
      VERIFY(m_hNewObj=CreateCompatibleBitmap(CompatibleToDC,w,h));
      VERIFY(m_hOldObj=SelectObject(m_hDC,m_hNewObj)); 
   }

   HBITMAP Old() const { return (HBITMAP)m_hOldObj; }
};


//******************************************************************
// Palettes
//******************************************************************

// This class is a bit different
// It cannot be used to create a palette, only to select it
class CSelPalette  : public CSelect
{
   // You need your own palette
   HPALETTE OldPalette;             

public:           
   
   CSelPalette(HDC hDC):CSelect(hDC),OldPalette(NULL){}

   CSelPalette(HDC hDC, HPALETTE hPalette, 
               BOOL ForceBackground=FALSE, BOOL Realize=TRUE)
     :CSelect(hDC)
   { 
      ASSERT(hPalette);
      ASSERT(GetDeviceCaps(m_hDC,RASTERCAPS)&RC_PALETTE);
      VERIFY(OldPalette=SelectPalette(m_hDC,hPalette,
                                      ForceBackground)); 
      if (Realize) RealizePalette(m_hDC);
   }

   ~CSelPalette()
   { 
      if (OldPalette) VERIFY(SelectPalette(m_hDC,OldPalette,FALSE)); 
   }

   UINT Select(HPALETTE hPalette, 
               BOOL ForceBackground=FALSE, BOOL Realize=TRUE)
   { 
      ASSERT(hPalette);
      ASSERT(GetDeviceCaps(m_hDC,RASTERCAPS)&RC_PALETTE);
      HPALETTE old=SelectPalette(m_hDC,hPalette,ForceBackground);
      ASSERT(old);
      if (!OldPalette) OldPalette=old;
      if (Realize) return RealizePalette(m_hDC);
      else         return 0;
   }

   void Restore()
   {  
      if (OldPalette) VERIFY(SelectPalette(m_hDC,OldPalette,FALSE)); 
      OldPalette=NULL;
   }

   HPALETTE Old() const  { return OldPalette; }
};


//******************************************************************
// Set and restore other characteristics of the DC (not GDI objects)
//******************************************************************


class CSelROP2 : public CSelect
{
   int m_OldRop;

  public:

   CSelROP2(HDC hDC,int drawMode):CSelect(hDC)
   { 
      VERIFY(m_OldRop=SetROP2(m_hDC,drawMode));
   }

   CSelROP2(HDC hDC):CSelect(hDC)
   { 
      VERIFY(m_OldRop=GetROP2(m_hDC)); 
   }

   ~CSelROP2()
   { 
      if (m_OldRop) VERIFY(SetROP2(m_hDC,m_OldRop)); 
   }

   void Select(int drawmode)
   { 
      int old=SetROP2(m_hDC,drawmode); ASSERT(old); 
      if (!m_OldRop) m_OldRop=old;
   }

   void Restore()
   { 
      if (m_OldRop) VERIFY(SetROP2(m_hDC,m_OldRop)); 
      m_OldRop=0;
   }

   int Old() const { return m_OldRop; }
};


class CSelBkMode : public CSelect
{
   int m_OldBkMode;

  public:

   CSelBkMode(HDC hDC,int BkMode):CSelect(hDC)
   { 
      VERIFY(m_OldBkMode=SetBkMode(m_hDC,BkMode));
   }

   CSelBkMode(HDC hDC):CSelect(hDC)
   { 
      VERIFY(m_OldBkMode=GetBkMode(m_hDC)); 
   }

   ~CSelBkMode()
   { 
      if (m_OldBkMode) VERIFY(SetBkMode(m_hDC,m_OldBkMode)); 
   }

   void Select(int bkmode)
   {
      int old=SetBkMode(m_hDC,bkmode); ASSERT(old); 
      if (!m_OldBkMode) m_OldBkMode=old;
   }

   void Restore()
   { 
     if (m_OldBkMode) VERIFY(SetBkMode(m_hDC,m_OldBkMode)); 
     m_OldBkMode=0;
   }

   int Old() const { return m_OldBkMode; }
};



class CSelBkColor : public CSelect
{
   COLORREF m_OldBkColor;

  public:

   CSelBkColor(HDC hDC,COLORREF BkColor):CSelect(hDC)
   { 
      m_OldBkColor=SetBkColor(m_hDC,BkColor);
   }

   CSelBkColor(HDC hDC):CSelect(hDC)
   { 
      m_OldBkColor=GetBkColor(m_hDC); 
   }

   ~CSelBkColor()
   { 
      SetBkColor(m_hDC,m_OldBkColor); 
   }

   void Select(COLORREF color)
   {  
      SetBkColor(m_hDC,color); 
   }

   void Restore()
   {
      SetBkColor(m_hDC,m_OldBkColor); 
   }

   COLORREF Old() const { return m_OldBkColor; }
};


class CSelTextColor : public CSelect
{
   COLORREF m_OldTextColor;

  public:

   CSelTextColor(HDC hDC,COLORREF TextColor)
     :CSelect(hDC)
   {
      m_OldTextColor=SetTextColor(m_hDC,TextColor);
   }

   CSelTextColor(HDC hDC):CSelect(hDC)
   {
      m_OldTextColor=GetTextColor(m_hDC); 
   }

   ~CSelTextColor()
   {
      SetTextColor(m_hDC,m_OldTextColor); 
   }

   void Select(COLORREF color)
   {
      SetTextColor(m_hDC,color); 
   }

   void Restore()
   { 
      SetTextColor(m_hDC,m_OldTextColor); 
   }

   COLORREF Old() const { return m_OldTextColor; }
};


class CSelTextAlign : public CSelect
{
   UINT m_OldTextAlign;

  public:

   CSelTextAlign(HDC hDC,UINT TextAlign):CSelect(hDC)
   {
      m_OldTextAlign=SetTextAlign(m_hDC,TextAlign);
   }

   CSelTextAlign(HDC hDC):CSelect(hDC)
   {
      m_OldTextAlign=GetTextAlign(m_hDC); 
   }

   ~CSelTextAlign()
   {
      SetTextAlign(m_hDC,m_OldTextAlign); 
   }

   void Select(UINT align)
   {
      SetTextAlign(m_hDC,align); 
   }

   void Restore()
   { 
      SetTextAlign(m_hDC,m_OldTextAlign); 
   }

   UINT Old() const { return m_OldTextAlign; }
};


class CSelMapMode : public CSelect
{
   int m_OldMapMode;

  public:

   CSelMapMode(HDC hDC,int MapMode):CSelect(hDC)
   {
      VERIFY(m_OldMapMode=SetMapMode(m_hDC,MapMode));
   }

   CSelMapMode(HDC hDC):CSelect(hDC)
   {
      VERIFY(m_OldMapMode=GetMapMode(m_hDC)); 
   }

   ~CSelMapMode()
   { 
      if (m_OldMapMode) VERIFY(SetMapMode(m_hDC,m_OldMapMode)); 
   }

   void Select(int mode)
   { 
      int old=SetMapMode(m_hDC,mode); ASSERT(old); 
      if (!m_OldMapMode) m_OldMapMode=old;
   }

   void Restore()
   {
      if (m_OldMapMode) VERIFY(SetMapMode(m_hDC,m_OldMapMode)); 
      m_OldMapMode=0;
   }

   UINT Old() const { return m_OldMapMode; }
};

//******************************************************************
// Class for saving the complete state of the DC
//******************************************************************

class CSaveDC : public CSelect
{
   int m_SavedDC;

  public:

   CSaveDC(HDC hDC):CSelect(hDC)
   { 
      VERIFY(m_SavedDC=SaveDC(m_hDC)); 
   }

   ~CSaveDC()
   { 
      if (m_SavedDC) VERIFY(RestoreDC(m_hDC,m_SavedDC)); 
   }

   void Restore()
   {
      if (m_SavedDC) VERIFY(RestoreDC(m_hDC,m_SavedDC)); 
      m_SavedDC=0;
   }

   int Old() const { return m_SavedDC; }
};

//******************************************************************
// Simple helper class to call BeginPaint/EndPaint
//******************************************************************

class CMyPaintDC
{
     HWND        m_hWnd;
     HDC         m_hDC;
     PAINTSTRUCT m_PS;
     
   public:

     CMyPaintDC(HWND hWnd):m_hWnd(hWnd)
     {
        m_hDC=BeginPaint(m_hWnd,&m_PS);
     }
     ~CMyPaintDC()
     {
        EndPaint (m_hWnd,&m_PS);
     }
     operator HDC() const { return m_hDC; }
};


#endif
